package CompilerProj.Parser;

import CompilerProj.Lexical.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;


/** This class represents the syntactic unit:
  **
  ** statement = method-call-stmt | assign-stmt | if-stmt | while-stmt | for-stmt
  *            | return-stmt
  **
  ** rewritten as:
  **
  ** statement = name-stmt | if-stmt | while-stmt | for-stmt | return-stmt       */

public class Statement extends SyntacticUnit {
    
    
    public static final TokenSet STARTS = oneOf(NameStmt.STARTS,IfStmt.STARTS,
                                WhileStmt.STARTS,ForStmt.STARTS,ReturnStmt.STARTS);
    
    
    public Statement ( TokenSet s ) {
        
        super(s);
        
    }; // constructor
    
    
    public void parse ( ) {
        
        check(oneOf(NameStmt.STARTS,IfStmt.STARTS,WhileStmt.STARTS,ForStmt.STARTS,
                                  ReturnStmt.STARTS),"Statement expected",stopSet);
        if      ( tokenIn(NameStmt.STARTS) ) {
            NameStmt aNameStmt = new NameStmt(stopSet);
            aNameStmt.parse();
        }
        else if ( tokenIn(IfStmt.STARTS) ) {
            IfStmt anIfStmt = new IfStmt(stopSet);
            anIfStmt.parse();
        }
        else if ( tokenIn(WhileStmt.STARTS) ) {
            WhileStmt aWhileStmt = new WhileStmt(stopSet);
            aWhileStmt.parse();
        }
        else if ( tokenIn(ForStmt.STARTS) ) {
            ForStmt aForStmt = new ForStmt(stopSet);
            aForStmt.parse();
        }
        else if ( tokenIn(ReturnStmt.STARTS) ) {
            ReturnStmt aReturnStmt = new ReturnStmt(stopSet);
            aReturnStmt.parse();
        };
        recognized("statement");
        
    }; // parse
    
    
} // Statement