package CompilerProj.Parser;

import CompilerProj.Lexical.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;


/** This class is an abstract supercalss of all classes representing
  * syntactic units in the language. It provides standard (helper) methods
  * in support of parsing. Every syntactic unit has a set of stop tokens
  * for parsing.                                                                 */

public abstract class SyntacticUnit {
    
    
    protected TokenSet  stopSet;
    
    
    protected SyntacticUnit ( TokenSet s ) { ... }; // constructor
    
    
    public abstract void parse ( ) ;
    
    
// Syntactic Analysis Helper Methods
    
    protected static boolean tokenIs ( TokenKind tk ) { ... }; // tokenIs
    
    
    protected static boolean tokenIn ( TokenSet set ) { ... }; // tokenIn
    
    
    protected void accept ( ) {
        
        source.nextToken();
        
    }; // accept
    
    
    protected void expect ( TokenKind expected, String errMsg, TokenSet context ) {
        
        if ( tokenIs(expected) ) {
            accept();
        }
        else {
            listing.writeError(...);
            skipTo(context.except(expected));
        };
        
    }; // expect
    
    
    protected void check ( TokenSet expect, String errMsg, TokenSet context ) {
        
        if ( ! tokenIn(expect) ) {
            listing.writeError(...);
            skipTo(oneOf(expect,context));
        };
        
    }; // check
    
    
    protected void recognized ( String syu ) {
        
        code.writeLine(syu+" recognized");
        
    };  // recognized
    
    
    private void skipTo ( TokenSet context ) {
        
        while ( ! tokenIn(context) ) {
            source.nextToken();
        };
        listing.writeMsg("Recovered to: "...);
        
    }; // skipTo
    
    
} // SyntacticUnit